"""

This module contains the MyProfile class and its methods.

"""
import asyncio
import json

from pyegeria import body_slimmer
from pyegeria._client import Client
from pyegeria._validators import validate_name, validate_search_string


class MyProfile(Client):
    """A class representing the profile of a user.

    This class provides methods for retrieving the profile details
    of a user associated with a token.

    Parameters
    ----------
    server_name : str
        The name of the view server to configure.
    platform_url : str
        The URL of the platform.
    token : str, optional
        The token associated with the user. Default is None.
    user_id : str, optional
        The user ID. Default is None.
    user_pwd : str, optional
        The user password. Default is None.
    sync_mode : bool, optional
        The flag indicating whether to use synchronous mode. Default
        is True.
    """

    def __init__(
            self,
            server_name: str,
            platform_url: str,
            token: str = None,
            user_id: str = None,
            user_pwd: str = None,
            sync_mode: bool = True
    ):

        Client.__init__(self, server_name, platform_url, user_id=user_id, user_pwd=user_pwd,
                        token=token, async_mode=sync_mode)
        self.my_profile_command_root: str = f"{platform_url}/servers"

    #
    #       MyProfile
    #

    async def _async_get_my_profile(self, server_name: str = None) -> dict | str:
        """ Get the profile of the user associated with the token used.

        Parameters
        ----------
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

        Profile details as a dict.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.my_profile_command_root}/{server_name}/api/open-metadata/my-profile"

        response = await self._async_make_request("GET", url)
        return response

    def get_my_profile(self, server_name: str = None) -> dict | str:
        """ Get the profile of the user associated with the token used.

                Parameters
                ----------
                server_name : str, optional
                    The name of the server to  configure.
                    If not provided, the server name associated with the instance is used.

                Returns
                -------
                List | str

                Profile details as a dict.

                Raises
                ------

                InvalidParameterException
                  If the client passes incorrect parameters on the request - such as bad URLs or invalid values
                PropertyServerException
                  Raised by the server when an issue arises in processing a valid request
                NotAuthorizedException
                  The principle specified by the user_id does not have authorization for the requested action
                """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_my_profile(server_name))

        return response.json().get("personalProfile", "No one found")

    async def _async_get_assigned_actions(self, actor_guid: str, status: str = "OPEN", server_name: str = None,
                                          start_from: int = 0, page_size: int = 100) -> list | str:
        """ Get assigned actions for the actor. Async version.

        Parameters
        ----------
        actor_guid: str
            The GUID of the actor whose assigned actions are to be retrieved.
        status: str
            The status of teh action to filter on. Default value is "OPEN".
        server_name: str, optional
            The name of the server. If not provided, the value of self.server_name will be used.
        start_from: int, optional
            The index from which to start retrieving the assigned actions. Default is 0.
        page_size: int, optional
            The number of assigned actions to retrieve per page. Default is 100.

        Returns
        -------
        list or str
            A list of assigned actions is returned. If there aren't any, a string is returned indicating that.

        Raises
        ------
        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        """
        if server_name is None:
            server_name = self.server_name

        if page_size is None:
            page_size = self.page_size

        body = {"status": status}

        url = (f"{self.my_profile_command_root}/{server_name}/api/open-metadata/my-profile/actors/{actor_guid}"
               f"/assigned/to-dos?startFrom={start_from}&pageSize={page_size}&")

        response = await self._async_make_request("POST", url, body)

        return response.json().get("elements", "No entries found")

    def get_assigned_actions(self, actor_guid: str, status: str = "OPEN", server_name: str = None, start_from: int = 0,
                             page_size: int = 100) -> list | str:
        """ Get assigned actions for the actor.
        Parameters
        ----------
        actor_guid: str
            The GUID of the actor whose assigned actions are to be retrieved.
        status: str
            The status of teh action to filter on. Default value is "OPEN".
        server_name: str, optional
            The name of the server. If not provided, the value of self.server_name will be used.
        start_from: int, optional
            The index from which to start retrieving the assigned actions. Default is 0.
        page_size: int, optional
            The number of assigned actions to retrieve per page. Default is 100.

        Returns
        -------
        list or str
            A list of assigned actions is returned. If there aren't any, a string is returned indicating that.

        Raises
        ------
        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_assigned_actions(actor_guid, status, server_name,
                                             start_from, page_size)
        )

        return response

    async def _async_get_actions_for_action_target(self, element_guid: str, status: str = "OPEN",
                                                   server_name: str = None,
                                                   start_from: int = 0, page_size: int = 100) -> list | str:
        """ Get actions assigned to the action target. Async version.

            Parameters
            ----------
            element_guid: str
                The GUID of the target whose assigned actions are to be retrieved.
            status: str
                The status of teh action to filter on. Default value is "OPEN".
            server_name: str, optional
                The name of the server. If not provided, the value of self.server_name will be used.
            start_from: int, optional
                The index from which to start retrieving the assigned actions. Default is 0.
            page_size: int, optional
                The number of assigned actions to retrieve per page. Default is 100.

            Returns
            -------
            list or str
                A list of assigned actions is returned. If there aren't any, a string is returned indicating that.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action
            """
        if server_name is None:
            server_name = self.server_name

        validate_name(element_guid)

        body = {"status": status}

        url = (f"{self.my_profile_command_root}/{server_name}/api/open-metadata/my-profile/elements/{element_guid}"
               f"/action-targets/to-dos?start-from={start_from}&page-size={page_size}")

        response = await self._async_make_request("POST", url, body)
        return response.json() if response is not None else "No Results"

    def get_actions_for_action_target(self, element_guid: str, status: str = "OPEN", server_name: str = None,
                                      start_from: int = 0, page_size: int = 100) -> list | str:
        """ Get actions assigned to the action target.

            Parameters
            ----------
            element_guid: str
                The GUID of the target whose assigned actions are to be retrieved.
            status: str
                The status of teh action to filter on. Default value is "OPEN"
            server_name: str, optional
                The name of the server. If not provided, the value of self.server_name will be used.
            start_from: int, optional
                The index from which to start retrieving the assigned actions. Default is 0.
            page_size: int, optional
                The number of assigned actions to retrieve per page. Default is 100.

            Returns
            -------
            list or str
                A list of assigned actions is returned. If there aren't any, a string is returned indicating that.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action
            """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_actions_for_action_target(element_guid, status, server_name,
                                                      start_from, page_size)
        )

        return response

    async def _async_get_actions_for_sponsor(self, element_guid: str, status: str = "", server_name: str = None,
                                             start_from: int = 0, page_size: int = 100) -> list | str:
        """ Get actions assigned to an owner. Async version.

            Parameters
            ----------
            element_guid: str
                The GUID of the target whose assigned actions are to be retrieved.
            status: str
                The status of the action to filter on. Default value is "OPEN".
            server_name: str, optional
                The name of the server. If not provided, the value of self.server_name will be used.
            start_from: int, optional
                The index from which to start retrieving the assigned actions. Default is 0.
            page_size: int, optional
                The number of assigned actions to retrieve per page. Default is 100.

            Returns
            -------
            list or str
                A list of assigned actions is returned. If there aren't any, a string is returned indicating that.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action
            """
        if server_name is None:
            server_name = self.server_name

        validate_name(element_guid)

        body = {"status": status}

        url = (f"{self.my_profile_command_root}/{server_name}/api/open-metadata/my-profile/elements/{element_guid}"
               f"/sponsored/to-dos?startFrom={start_from}&pageSize={page_size}")

        response = await self._async_make_request("POST", url, body)
        return response.json() if response is not None else "No Results"

    def get_actions_for_sponsor(self, element_guid: str, status: str = "OPEN", server_name: str = None,
                                start_from: int = 0, page_size: int = 100) -> list | str:
        """ Get actions assigned to an owner.
            Parameters
            ----------
            element_guid: str
                The GUID of the target whose assigned actions are to be retrieved.
            status: str
                The status of teh action to filter on. Default value is "OPEN".
            server_name: str, optional
                The name of the server. If not provided, the value of self.server_name will be used.
            start_from: int, optional
                The index from which to start retrieving the assigned actions. Default is 0.
            page_size: int, optional
                The number of assigned actions to retrieve per page. Default is 100.

            Returns
            -------
            list or str
                A list of assigned actions is returned. If there aren't any, a string is returned indicating that.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action
            """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_actions_for_sponsor(
            element_guid, status, server_name, start_from, page_size)
        )
        return response

    async def _async_create_to_do(self, body: dict, server_name: str = None) -> str:
        """ Create a To-Do item. Async version.
        Parameters
        ----------
        body : dict
                    The dictionary containing the details of the to-do item.
        server_name : str, optional
            The name of the server where the to-do item will be created. If not provided,
            the default server name associated with the instance of the class will be used.

        Returns
        -------
        None
            This method does not return any value.

        Raises
        ------
        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.my_profile_command_root}/{server_name}/api/open-metadata/my-profile/to-dos"
        response = await self._async_make_request("POST", url, body)
        return response.json().get("guid", "No guid returned")

    def create_to_do(self, body: dict, server_name: str = None) -> str:
        """ Create a To-Do item.
            Parameters
            ----------
            body : dict
                The dictionary containing the details of the to-do item.
            server_name : str, optional
                The name of the server where the to-do item will be created. If not provided,
                the default server name associated with the instance of the class will be used.

            Returns
            -------
            None
                This method does not return any value.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

            Notes
            -----

            Here is a typical body:
            body = {
                    "properties": {
                        "class" : "ToDoProperties",
                        "qualifiedName": f"Test-To-Do-{time.asctime()}",
                        "name": to_do,
                        "description": to_do_desc,
                        "toDoType": to_do_type,
                        "priority": 0,
                        "dueTime": "2024-03-11T15:42:11.307Z",
                        "status": "OPEN"
                    },
                "assignToActorGUID": erins_guid
        }
            """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_create_to_do(body, server_name)
        )
        return response

    async def _async_get_to_do(self, todo_guid: str, server_name: str = None) -> dict | str:
        """ Get a To-Do item. Async version.
           Parameters
           ----------
           todo_guid: str
                Identifier of the To-Do item.
           server_name : str, optional
               The name of the server where the to-do item will be created. If not provided,
               the default server name associated with the instance of the class will be used.

           Returns
           -------
           None
               This method does not return any value.

           Raises
           ------
           InvalidParameterException
             If the client passes incorrect parameters on the request - such as bad URLs or invalid values
           PropertyServerException
             Raised by the server when an issue arises in processing a valid request
           NotAuthorizedException
             The principle specified by the user_id does not have authorization for the requested action
           """
        if server_name is None:
            server_name = self.server_name

        validate_name(todo_guid)

        url = f"{self.my_profile_command_root}/{server_name}/api/open-metadata/my-profile/to-dos/{todo_guid}"

        response = await self._async_make_request("GET", url)
        # return response.text if response is not None else "No Results"
        return json.loads(response.text).get("elements", "No TODO returned")

    def get_to_do(self, todo_guid: str, server_name: str = None) -> dict | str:
        """ Get a To-Do item. Async version.
           Parameters
           ----------
           todo_guid: str
                Identifier of the To-Do item.
           server_name : str, optional
               The name of the server where the to-do item will be created. If not provided,
               the default server name associated with the instance of the class will be used.

           Returns
           -------
           None
               This method does not return any value.

           Raises
           ------
           InvalidParameterException
             If the client passes incorrect parameters on the request - such as bad URLs or invalid values
           PropertyServerException
             Raised by the server when an issue arises in processing a valid request
           NotAuthorizedException
             The principle specified by the user_id does not have authorization for the requested action
           """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_to_do(todo_guid, server_name))

        return response

    async def _async_update_to_do(self, todo_guid: str, body: dict, is_merge_update: bool = True,
                                  server_name: str = None) -> None:
        """ Update a To-Do item. Async version.
            Parameters
            ----------
            todo_guid: str
              Identifier of the To-Do item.
            body: str
                The details to update the to-do item with.
            server_name : str, optional
             The name of the server where the to-do item will be created. If not provided,
             the default server name associated with the instance of the class will be used.

            Returns
            -------
            None
             This method does not return any value.

            Raises
            ------
            InvalidParameterException
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
            Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
            The principle specified by the user_id does not have authorization for the requested action
            """
        if server_name is None:
            server_name = self.server_name

        is_merge_update_t = str(is_merge_update).lower()

        validate_name(todo_guid)

        url = (f"{self.my_profile_command_root}/{server_name}/api/open-metadata/my-profile/to-dos/"
               f"{todo_guid}?isMergeUpdate={is_merge_update_t}")

        await self._async_make_request("POST", url, body)
        return

    def update_to_do(self, todo_guid: str, body: dict, is_merge_update: bool = True,
                     server_name: str = None) -> None:
        """ Update a To-Do item.
            Parameters
            ----------
            todo_guid: str
              Identifier of the To-Do item.
            body: str
                The details to update the to-do item with.
            is_merge_update: bool [default: True]
                If true then merges the updated information, otherwise replace the existing information.
            server_name : str, optional
             The name of the server where the to-do item will be created. If not provided,
             the default server name associated with the instance of the class will be used.

            Returns
            -------
            None
             This method does not return any value.

            Raises
            ------
            InvalidParameterException
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
            Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
            The principle specified by the user_id does not have authorization for the requested action
            """

        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_update_to_do(todo_guid, body, is_merge_update, server_name))
        return

    async def _async_delete_to_do(self, todo_guid: str, server_name: str = None) -> None:
        """ Delete a To-Do item. Async version.
            Parameters
            ----------
            todo_guid: str
              Identifier of the To-Do item.
            server_name : str, optional
             The name of the server where the to-do item will be created. If not provided,
             the default server name associated with the instance of the class will be used.

            Returns
            -------
            None
             This method does not return any value.

            Raises
            ------
            InvalidParameterException
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
            Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
            The principle specified by the user_id does not have authorization for the requested action
            """
        if server_name is None:
            server_name = self.server_name

        validate_name(todo_guid)


        url = f"{self.my_profile_command_root}/{server_name}/api/open-metadata/my-profile/to-dos/{todo_guid}/delete"

        await self._async_make_request("POST", url)
        return

    def delete_to_do(self, todo_guid: str, server_name: str = None) -> None:
        """ Delete a To-Do item.
            Parameters
            ----------
            todo_guid: str
              Identifier of the To-Do item.
            server_name : str, optional
             The name of the server where the to-do item will be created. If not provided,
             the default server name associated with the instance of the class will be used.

            Returns
            -------
            None
             This method does not return any value.

            Raises
            ------
            InvalidParameterException
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
            Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
            The principle specified by the user_id does not have authorization for the requested action
            """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_delete_to_do(todo_guid, server_name))
        return

    async def _async_reassign_to_do(self, todo_guid: str, actor_guid: str, status: str = "OPEN",
                                    server_name: str = None) -> None:
        """ Reassign a To-Do item. Async version.
           Parameters
           ----------
            todo_guid: str
                Identifier of the To-Do item.
            actor_guid: str
                The actor to receive the reassigned to-do item.
            status: str [default = "OPEN"]
                Filter items to match this status.
            server_name : str, optional
                The name of the server where the to-do item will be created. If not provided,
                the default server name associated with the instance of the class will be used.

           Returns
           -------
           None
            This method does not return any value.

           Raises
           ------
           InvalidParameterException
           If the client passes incorrect parameters on the request - such as bad URLs or invalid values
           PropertyServerException
           Raised by the server when an issue arises in processing a valid request
           NotAuthorizedException
           The principle specified by the user_id does not have authorization for the requested action
           """
        if server_name is None:
            server_name = self.server_name

        validate_name(todo_guid)
        validate_name(actor_guid)
        body = {"status": status}

        url = (f"{self.my_profile_command_root}/{server_name}/api/open-metadata/my-profile/to-dos/"
               f"{todo_guid}/reassign/{actor_guid}")

        await self._async_make_request("POST", url, body)
        return

    def reassign_to_do(self, todo_guid: str, actor_guid: str, status: str = "OPEN", server_name: str = None) -> None:
        """ Reassign a To-Do item.
           Parameters
           ----------
            todo_guid: str
                Identifier of the To-Do item.
            actor_guid: str
                The actor to receive the reassigned to-do item.
            status: str [default = "OPEN"]
                Filter items to match this status.
            server_name : str, optional
                The name of the server where the to-do item will be created. If not provided,
                the default server name associated with the instance of the class will be used.

           Returns
           -------
           None
            This method does not return any value.

           Raises
           ------
           InvalidParameterException
           If the client passes incorrect parameters on the request - such as bad URLs or invalid values
           PropertyServerException
           Raised by the server when an issue arises in processing a valid request
           NotAuthorizedException
           The principle specified by the user_id does not have authorization for the requested action
           """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_reassign_to_do(todo_guid, actor_guid, status, server_name))
        return

    async def _async_find_to_do(self, search_string: str = "*", server_name: str = "None", status: str = None,
                                starts_with: bool = False, ends_with: bool = False, ignore_case: bool = True,
                                start_from: int = 0, page_size: int = 100) -> list | str:
        """ find To-Do items. Async version.
              Parameters
              ----------
                search_string: str
                   String to search against. If '*' then all to-do items will match.
                server_name : str, optional
                   The name of the server where the to-do item will be created. If not provided,
                   the default server name associated with the instance of the class will be used.
                status: str
                   Filter items to match this status. Defaults to "OPEN"
                starts_with : bool, [default=False], optional
                   Starts with the supplied string.
                ends_with : bool, [default=False], optional
                       Ends with the supplied string
                ignore_case : bool, [default=False], optional
                       Ignore case when searching
              Returns
              -------
              None
               List of To-Do items that match the criteria

              Raises
              ------
              InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
              PropertyServerException
              Raised by the server when an issue arises in processing a valid request
              NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action
              """
        if server_name is None:
            server_name = self.server_name

        starts_with_s = str(starts_with).lower()
        ends_with_s = str(ends_with).lower()
        ignore_case_s = str(ignore_case).lower()

        if search_string == '*':
            search_string = " "

        body = {
            "class": "ToDoStatusSearchString",
            "toDoStatus": status,
            "searchString": search_string
        }

        validate_search_string(search_string)

        url = (f"{self.my_profile_command_root}/{server_name}/api/open-metadata/my-profile/to-dos/"
               f"find-by-search-string?startFrom={start_from}&pageSize={page_size}&"
               f"startsWith={starts_with_s}&endsWith={ends_with_s}&ignoreCase={ignore_case_s}")

        response = await self._async_make_request("POST", url, body_slimmer(body))
        # return response.text
        return response.json().get("elements", "No ToDos found")

    def find_to_do(self, search_string: str, server_name: str = None, status: str = "OPEN",
                   starts_with: bool = False, ends_with: bool = False, ignore_case: bool = True,
                   start_from: int = 0, page_size: int = 100) -> list | str:
        """ find To-Do items.
            Parameters
            ----------
           search_string: str
              String to search against. If '*' then all to-do items will match.
           server_name : str, optional
              The name of the server where the to-do item will be created. If not provided,
              the default server name associated with the instance of the class will be used.
           status: str
              Filter items to match this status. Defaults to "OPEN"
           starts_with : bool, [default=False], optional
              Starts with the supplied string.
           ends_with : bool, [default=False], optional
                  Ends with the supplied string
           ignore_case : bool, [default=False], optional
                  Ignore case when searching
            start_from: int, [default=0], optional
                When multiple pages of results are available, the page number to start from.
            page_size: int, [default=None]
                The number of items to return in a single page. If not specified, the default will be taken from
                the class instance.
         Returns
         -------
         None
          List of To-Do items that match the criteria

         Raises
         ------
         InvalidParameterException
         If the client passes incorrect parameters on the request - such as bad URLs or invalid values
         PropertyServerException
         Raised by the server when an issue arises in processing a valid request
         NotAuthorizedException
         The principle specified by the user_id does not have authorization for the requested action
         """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_find_to_do(search_string, server_name, status, starts_with,
                                                                  ends_with, ignore_case, start_from, page_size))
        return response

    async def _async_get_to_dos_by_type(self, todo_type: str, server_name: str = None, status: str = "OPEN",
                                        start_from: int = 0, page_size: int = 100) -> list | str:
        """ Get To-Do items by type. Async version
            Parameters
            ----------
            todo_type: str
              Type of to-do to find
            server_name : str, optional
              The name of the server where the to-do item will be created. If not provided,
              the default server name associated with the instance of the class will be used.
            status: str
              Filter items to match this status. Defaults to "OPEN"
            start_from: int, [default=0], optional
                When multiple pages of results are available, the page number to start from.
            page_size: int, [default=None]
                The number of items to return in a single page. If not specified, the default will be taken from
                the class instance.
            Returns
            -------
            List of To-Do items that match the criteria

            Raises
            ------
            InvalidParameterException
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
            Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
            The principle specified by the user_id does not have authorization for the requested action
            """
        if server_name is None:
            server_name = self.server_name

        validate_name(todo_type)
        body = {
            "status": status,

        }

        url = (f"{self.my_profile_command_root}/{server_name}/api/open-metadata/my-profile/to-dos/types/"
               f"{todo_type}?startFrom={start_from}&pageSize={page_size}")

        response = await self._async_make_request("POST", url, body)
        return response.json().get("elements", "No ToDos found")

    def get_to_dos_by_type(self, todo_type: str, server_name: str = None, status: str = "OPEN",
                           start_from: int = 0, page_size: int = 100) -> list | str:
        """ Get To-Do items by type.
            Parameters
            ----------
            todo_type: str
              Type of to-do to find
            server_name : str, optional
              The name of the server where the to-do item will be created. If not provided,
              the default server name associated with the instance of the class will be used.
            status: str
              Filter items to match this status. Defaults to "OPEN"
            start_from: int, [default=0], optional
                When multiple pages of results are available, the page number to start from.
            page_size: int, [default=None]
                The number of items to return in a single page. If not specified, the default will be taken from
                the class instance.
            Returns
            -------
            List of To-Do items that match the criteria

            Raises
            ------
            InvalidParameterException
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
            Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
            The principle specified by the user_id does not have authorization for the requested action
            """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_to_dos_by_type(todo_type, server_name, status,
                                                                          start_from, page_size))
        return response

    async def _async_update_action_target_properties(self, action_target_guid: str, body: dict,
                                                     is_merge_update: bool = True, server_name: str = None) -> None:
        """ Get To-Do items by type. Async version
           Parameters
           ----------
            action_target_guid: str
                Identity of the action target to update.
            body: dict
                Details of the updates to make.
            is_merge_update : bool, [default=True], optional
                indicates if the update should be a merge or replacement.
            server_name : str, optional
              The name of the server where the to-do item will be created. If not provided,
              the default server name associated with the instance of the class will be used.
           Returns
           -------
            None

           Raises
           ------
            InvalidParameterException
                If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
                Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
                The principle specified by the user_id does not have authorization for the requested action
           """
        if server_name is None:
            server_name = self.server_name

        is_merge_update_t = str(is_merge_update).lower()

        validate_name(action_target_guid)

        url = (f"{self.my_profile_command_root}/{server_name}/api/open-metadata/my-profile/to-dos/"
               f"action-targets/{action_target_guid}?isMergeUpdate={is_merge_update_t}")

        await self._async_make_request("POST", url, body)
        return

    def update_action_target_properties(self, action_target_guid: str, body: dict,
                                        is_merge_update: bool = True, server_name: str = None) -> None:
        """ Get To-Do items by type.
           Parameters
           ----------
            action_target_guid: str
                Identity of the action target to update.
            body: dict
                Details of the updates to make.
            is_merge_update : bool, [default=True], optional
                indicates if the update should be a merge or replacement.
            server_name : str, optional
              The name of the server where the to-do item will be created. If not provided,
              the default server name associated with the instance of the class will be used.
           Returns
           -------
            None

           Raises
           ------
            InvalidParameterException
                If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
                Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
                The principle specified by the user_id does not have authorization for the requested action
           """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_update_action_target_properties(action_target_guid,
                                                                            body, is_merge_update,
                                                                            server_name))
        return




"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

This sample creates the collection structure show in https://egeria-project.org/types/0/0021-Collections

"""

import time

from rich import print
from rich.console import Console

from pyegeria import CollectionManager, InvalidParameterException, PropertyServerException, UserNotAuthorizedException

view_server = 'view-server'
platform_url = 'https://localhost:9443'
user = 'erinoverview'
console = Console(width=200)

try:
    c_client = CollectionManager(view_server, platform_url,
                                 user_id=user)

    token = c_client.create_egeria_bearer_token(user, "secret")
    start_time = time.perf_counter()

    # Create a Data Products Root collection
    anchor_guid = None
    parent_guid = None
    parent_relationship_type_name = None
    parent_at_end1 = False
    display_name = "Digital Products Root"
    description = "This is the root catalog for digital products"
    collection_type = "Digital Products Root"
    is_own_anchor = True

    response = c_client.create_root_collection(anchor_guid, parent_guid,
                                               parent_relationship_type_name, parent_at_end1,
                                               display_name, description, collection_type,
                                               is_own_anchor)
    # Create first folder for Agriculture Insights
    parent_guid = response
    parent_relationship_type_name = "CollectionMembership"
    display_name = "Agriculture Insights Collection"
    description = "A folder for agricultural insights data product collections"
    collection_type = "Digital Product Marketplace"

    folder1 = c_client.create_folder_collection(None, parent_guid,
                                                parent_relationship_type_name,
                                                True, display_name, description,
                                                collection_type, True, "DATE_CREATED",
                                                None
                                                )
    print(f"\n\n created a folder with guid {folder1}")
    # create second folder for Earth Observations
    display_name = "Earth Observation Data Collection"
    description = "A folder for Earth Observation data product collections"

    folder2 = c_client.create_folder_collection(None, parent_guid,
                                                parent_relationship_type_name,
                                                True, display_name, description,
                                                collection_type, True, "DATE_CREATED",
                                                None
                                                )
    print(f"\n\n created a folder with guid {folder2}")

    # create a digital product, child of folder 1, for Land Use products
    parent_guid = folder1
    parent_relationship_type_name = "CollectionMembership"
    display_name = "Land Use Classification"
    description = "Land use classification assets"
    collection_type = "Digital Product"
    classification_name = "Digital Product"
    body_3 = {
            "class": "NewDigitalProductRequestBody",
            "isOwnAnchor": True,
            "parentGUID": parent_guid,
            "parentRelationshipTypeName": parent_relationship_type_name,
            "parentAtEnd1": True,
            "collectionProperties": {
                "class": "CollectionProperties",
                "qualifiedName": f"{classification_name}-{display_name}-{time.asctime()}",
                "name": display_name,
                "description": description,
                "collectionType": collection_type,
                "collectionOrdering": "DATE_CREATED",
            },
            "digitalProductProperties": {
                "class": "DigitalProductProperties",
                "productStatus": "ACTIVE",
                "productName": "Land Use Classifications",
                "productType": "Geospatial Data Assets",
                "description": "Land use classification assets",
                "introductionDate": "2023-12-01",
                "maturity": "Nacent",
                "serviceLife": "3 years",
                "currentVersion": "V.5",
                "nextVersionDate": "2024-12-01",
                "withdrawDate": "2030-01-01",
                "additionalProperties": {
                    "thought_id": "a guid",
                    "license": "cc-by-sa",
                }
            }
    }
    folder3 = c_client.create_digital_product(body_3)
    print(f"\n\n created a collection with guid {folder3}")

    # create a fourth collection, a digital product, child of folder 2, for Landsat 8
    parent_guid = folder2
    display_name = "Landsat 8"
    description = "Landsat 8 data products"

    body_4 = {
            "class": "NewDigitalProductRequestBody",
            "isOwnAnchor": True,
            "parentGUID": parent_guid,
            "parentRelationshipTypeName": parent_relationship_type_name,
            "parentAtEnd1": True,
            "collectionProperties": {
                "class": "CollectionProperties",
                "qualifiedName": f"{classification_name}-{display_name}-{time.asctime()}",
                "name": display_name,
                "description": description,
                "collectionType": collection_type,
                "collectionOrdering": "DATE_CREATED",
            },
            "digitalProductProperties": {
                "class": "DigitalProductProperties",
                "productStatus": "ACTIVE",
                "productName": "Landsat 8 Imagery",
                "productType": "Geospatial Data Assets",
                "description": description,
                "introductionDate": "2024-01-01",
                "maturity": "Mature",
                "serviceLife": "3 years",
                "currentVersion": "V1.5",
                "nextVersion": "2024-06-01",
                "withdrawDate": "2030-01-01",
                "additionalProperties": {
                    "thought_id": "a guid",
                    "license": "cc-by-sa",
                }
            }
    }
    folder4 = c_client.create_digital_product(body_4)
    print(f"\n\n created a collection with guid {folder4}")

    # Now create a 5th collection for sentinel 2 data
    parent_guid = folder2
    display_name = "Sentinel 2"
    description = "Sentinel 2 products"
    parent_relationship_type_name = "CollectionMembership"
    collection_type = "Digital Product Marketplace"

    folder5 = c_client.create_folder_collection(None, parent_guid,
                                                parent_relationship_type_name,
                                                True, display_name, description,
                                                collection_type, True, "DATE_CREATED",
                                                None
                                                )
    # Create a DigitalProduct for Level-1B
    parent_guid = folder5
    display_name = "Sentinel 2 - Level 1B"
    description = "Level 1B of Sentinel 2"

    body_6 = {
        "class": "NewDigitalProductRequestBody",
        "anchor_guid": parent_guid,
        "isOwnAnchor": False,
        "parentGUID": parent_guid,
        "parentRelationshipTypeName": parent_relationship_type_name,
        "parentAtEnd1": True,
        "collectionProperties": {
            "class": "CollectionProperties",
            "qualifiedName": f"{classification_name}-{display_name}-{time.asctime()}",
            "name": display_name,
            "description": description,
            "collectionType": collection_type,
            "collectionOrdering": "DATE_CREATED",
        },
        "digitalProductProperties": {
            "class": "DigitalProductProperties",
            "productStatus": "ACTIVE",
            "productName": "Sentinel 2 - Level 1B",
            "productType": "Geospatial Data Assets",
            "description": description,
            "introductionDate": "2024-01-01",
            "maturity": "Mature",
            "serviceLife": "3 years",
            "currentVersion": "V1.5",
            "nextVersion": "2024-06-01",
            "withdrawDate": "2030-01-01",
            "additionalProperties": {
                "thought_id": "a guid",
                "license": "cc-by-sa",
            }
        }
    }
    folder6 = c_client.create_digital_product(body_6)
    print(f"\n\n created a collection with guid {folder6}")

    # now lets create a digital product for - Level - 1c
    parent_guid = folder5
    display_name = "Sentinel 2 - Level 1C"
    description = "Level 1C of Sentinel 2"
    body_7 = {
        "class": "NewDigitalProductRequestBody",
        "anchor_guid": parent_guid,
        "isOwnAnchor": False,
        "parentGUID": parent_guid,
        "parentRelationshipTypeName": parent_relationship_type_name,
        "parentAtEnd1": True,
        "collectionProperties": {
            "class": "CollectionProperties",
            "qualifiedName": f"{classification_name}-{display_name}-{time.asctime()}",
            "name": display_name,
            "description": description,
            "collectionType": collection_type,
            "collectionOrdering": "DATE_CREATED",
        },
        "digitalProductProperties": {
            "class": "DigitalProductProperties",
            "productStatus": "ACTIVE",
            "productName": "Sentinel 2 - Level 1B",
            "productType": "Geospatial Data Assets",
            "description": description,
            "introductionDate": "2024-01-01",
            "maturity": "Mature",
            "serviceLife": "3 years",
            "currentVersion": "V1.5",
            "nextVersion": "2024-06-01",
            "withdrawDate": "2030-01-01",
            "additionalProperties": {
                "thought_id": "a guid",
                "license": "cc-by-sa",
            }
        }
    }
    folder7 = c_client.create_digital_product(body_7)
    print(f"\n\n created a collection with guid {folder7}")

    # now let's create a digital product for - Level - 2A
    parent_guid = folder5
    display_name = "Sentinel 2 - Level 2A"
    description = "Level 2A of Sentinel 2"
    body_8 = {
        "class": "NewDigitalProductRequestBody",
        "anchor_guid": parent_guid,
        "isOwnAnchor": False,
        "parentGUID": parent_guid,
        "parentRelationshipTypeName": parent_relationship_type_name,
        "parentAtEnd1": True,
        "collectionProperties": {
            "class": "CollectionProperties",
            "qualifiedName": f"{classification_name}-{display_name}-{time.asctime()}",
            "name": display_name,
            "description": description,
            "collectionType": collection_type,
            "collectionOrdering": "DATE_CREATED",
        },
        "digitalProductProperties": {
            "class": "DigitalProductProperties",
            "productStatus": "ACTIVE",
            "productName": "Sentinel 2 - Level 1B",
            "productType": "Geospatial Data Assets",
            "description": description,
            "introductionDate": "2024-01-01",
            "maturity": "Mature",
            "serviceLife": "3 years",
            "currentVersion": "V1.5",
            "nextVersion": "2024-06-01",
            "withdrawDate": "2030-01-01",
            "additionalProperties": {
                "thought_id": "a guid",
                "license": "cc-by-sa",
            }
        }
    }
    folder8 = c_client.create_digital_product(body_8)
    print(f"\n\n created a collection with guid {folder8}")

except (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException
) as e:
    console.print_exception(show_locals=True)

finally:
    c_client.close_session()



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

 Governance Engine functions.  These functions initiate and manage Governance Actions and Processes

 This module is deprecated - please use Automated Curation.
"""

from datetime import datetime

from ._client import Client
from ._globals import enable_ssl_check


def body_slimmer(body: dict) -> dict:
    """ body_slimmer is a little function to remove unused keys from a dict

    Parameters
    ----------
    body : the dictionary that you want to slim

    Returns
    -------
    dict:
        a slimmed body
    """
    slimmed = {key: value for key, value in body.items() if value}
    return slimmed


class GovEng(Client):
    """
       Client to initiate and manage governance actions and processes.

       Attributes:
           server_name: str
                Name of the server to use.
           platform_url : str
               URL of the server platform to connect to
           user_id : str
               The identity of the user calling the method - this sets a default optionally used by the methods
               when the user doesn't pass the user_id on a method call.
           user_pwd: str = None
                The password associated with the user
           verify_flag: bool = enable_ssl_check
                Set true for SSL verification to be enabled, false for disabled. Default behaviour set by the
                enable_ssl_check attribute from _globals.py

       """

    def __init__(
            self,
            server_name: str,
            platform_url: str,
            user_id: str,
            user_pwd: str = None,
            verify_flag: bool = enable_ssl_check,
    ):
        self.admin_command_root: str
        Client.__init__(self, server_name, platform_url, user_id,
                        user_pwd, verify_flag)
        self.engine_command_root = (
                self.platform_url
                + "/servers/" + server_name
                + "/open-metadata/framework-services/governance-engine/open-governance-service/users/"
                + user_id
        )

    def get_engine_actions(self, start_from: int = 0, page_size: int = 0) -> [dict]:
        """ Get engine actions associated deployed on the server.

        Args:
            start_from (int, optional): The index to start retrieving processes from. Defaults to 0.
            page_size (int, optional): The number of processes to retrieve per page. Defaults to 0 (no pagination).

        Returns:
            List[str]: A list of JSON representations of governance action processes matching the provided name.

        Raises:
            InvalidParameterException: If the API response indicates an error (non-200 status code),
            this exception is raised with details from the response content.

        Note:
            Pagination of 0 defaults to server default.

        """
        url = (self.engine_command_root + "/engine-actions?startFrom=" +
               str(start_from) + "&pageSize=" + str(page_size))
        response = self.make_request("GET", url)

        governance_elements = response.json().get('elements')
        return governance_elements

    def get_engine_action(self, engine_action_guid: str) -> str:
        """ Return the governance action associated with the supplied guid

        Parameters
        ----------
        engine_action_guid : str
            The unique identifier of the engine action to retrieve.
        Returns
        -------
        JSON string of the governance action associated with the guid

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException
        """

        url = self.engine_command_root + "/engine-actions/" + engine_action_guid
        response = self.make_request("GET", url)

        governance_element = response.json().get('element')
        return governance_element

    def get_active_engine_actions(self, start_from: int = 0, page_size: int = 0) -> [str]:
        """ Get active governance actions associated on the server.

        Args:
            start_from (int, optional): The index to start retrieving processes from. Defaults to 0.
            page_size (int, optional): The number of processes to retrieve per page. Defaults to 0 (no pagination).

        Returns:
            List[str]: A list of JSON representations of governance action processes matching the provided name.

        Raises:
            InvalidParameterException: If the API response indicates an error (non-200 status code),
            this exception is raised with details from the response content.

        Note:
            Pagination of 0 defaults to server default.
        """
        url = (self.engine_command_root + "/engine-actions/active?startFrom=" +
               str(start_from) + "&pageSize=" + str(page_size))

        response = self.make_request("GET", url)
        governance_elements = response.json().get('elements')
        return governance_elements
        
    def get_engine_actions_by_name(self, name: str, start_from: int = 0, page_size: int = 0) -> str:
        """ Retrieve engine actions matching the name string.
        Args:
            name (str): The qualified name or display name of the governance action to get.
            start_from (int, optional): The index to start retrieving processes from. Defaults to 0.
            page_size (int, optional): The number of processes to retrieve per page. Defaults to 0 (no pagination).

        Returns:
            List[str]: A list of JSON representations of governance action processes matching the provided name.

        Raises:
            InvalidParameterException: If the API response indicates an error (non-200 status code),
            this exception is raised with details from the response content.

        Note:
            Pagination of 0 defaults to server default.

        """
        url = (self.engine_command_root + "/engine-actions/by-name?startFrom=" +
               str(start_from) + "&pageSize=" + str(page_size))
        body = {
            "class": "NameRequestBody",
            "name": name
        }
        response = self.make_request("POST", url, body)

        governance_elements = response.json().get('elements')
        return governance_elements

    def find_engine_actions(self, search_string: str, start_from: int = 0, page_size: int = 0) -> [str]:
        """ Search for engine actions matching the search string.

        Args:
            search_string (str): The search string to query for.
            start_from (int, optional): The index to start retrieving processes from. Defaults to 0.
            page_size (int, optional): The number of processes to retrieve per page. Defaults to 0 (no pagination).

        Returns:
            List[str]: A list of JSON representations of governance action processes matching the provided name.

        Raises:
            InvalidParameterException: If the API response indicates an error (non-200 status code),
            this exception is raised with details from the response content.

        Note:
            Pagination of 0 defaults to server default.
       """
        url = (self.engine_command_root + "/engine-actions/by-search-string?startFrom=" +
               str(start_from) + "&pageSize=" + str(page_size))
        body = {
            "class": "SearchStringRequestBody",
            "searchString": search_string
        }
        response = self.make_request("POST", url, body)

        governance_elements = response.json().get('elements')
        return governance_elements

    def get_governance_action_process_by_guid(self, gov_process_guid: str) -> str:
        """
            Retrieves information about a governance action process based on its GUID.

            Args:
                gov_process_guid (str): The GUID (Globally Unique Identifier) of the governance action process.

            Returns:
                str: The JSON representation of the governance action process element.

            Raises:
                InvalidParameterException: If the API response indicates an error (non-200 status code),
                this exception is raised with details from the response content.
                PropertyServerException: If the API response indicates a server side error.
                UserNotAuthorizedException:

            Note:
                This method assumes that the provided GUID is valid and corresponds to an existing
                governance action process in the system.

            """
        url = self.engine_command_root + "/governance-action-processes/" + gov_process_guid
        response = self.make_request("GET", url)
        governance_element = response.json().get('element')
        return governance_element

    def get_governance_action_processes_by_name(self, name: str, start_from: int = 0, page_size: int = 0) -> [str]:
        """
        Retrieves governance action processes based on their name only (no wildcards).

        Args:
            name (str): The qualified name or display name of the governance action to get.
            start_from (int, optional): The index to start retrieving processes from. Defaults to 0.
            page_size (int, optional): The number of processes to retrieve per page. Defaults to 0 (no pagination).

        Returns:
            List[str]: A list of JSON representations of governance action processes matching the provided name.

        Raises:
            InvalidParameterException: If the API response indicates an error (non-200 status code),
            this exception is raised with details from the response content.

        Note:
            Pagination of 0 defaults to server default.

        """
        url = (self.engine_command_root + "/governance-action-processes/by-name?startFrom=" + str(start_from)
               + "&pageSize=" + str(page_size))
        body = {
            "class":          "NameRequestBody",
            "name":           name
        }
        response = self.make_request("POST", url, body)
        governance_elements = response.json().get('elements')
        return governance_elements
        
    def find_governance_action_processes(self, search_string: str, start_from: int = 0, page_size: int = 0) -> [str]:
        """ Return governance action processes that match the search string (with regex).

        Args:
            search_string (str): The search string to query for.
            start_from (int, optional): The index to start retrieving processes from. Defaults to 0.
            page_size (int, optional): The number of processes to retrieve per page. Defaults to 0 (no pagination).

        Returns:
            List[str]: A list of JSON representations of governance action processes matching the provided name.

        Raises:
            InvalidParameterException: If the API response indicates an error (non-200 status code),
            this exception is raised with details from the response content.

        Note:
            Pagination of 0 defaults to server default.

        """
        url = (self.engine_command_root + "/governance-action-processes/by-search-string?startFrom=" + str(start_from)
               + "&pageSize=" + str(page_size))
        body = {
            "class": "SearchStringRequestBody",
            "searchString": search_string
        }
        response = self.make_request("POST", url, body)

        governance_elements = response.json().get('elements')
        return governance_elements
        
    def initiate_governance_action_process(self, qualified_name: str, request_source_guids: [str],
                                           action_targets: [str], start_time: datetime, request_parameters: dict,
                                           orig_service_name: str, orig_engine_name: str) -> str:
        """ initiate_gov_action_process

        This method starts a governance action process using the supplied parameters.

        Parameters
        ----------
        qualified_name: str
            - unique name for the governance process
        request_source_guids: [str]
            - request source elements for the resulting governance action service
        action_targets: [str]
            -list of action target names to GUIDs for the resulting governance action service
        start_time: datetime
            - time to start the process
        request_parameters: [str]
            - parameters passed into the process
        orig_service_name: str
            - unique name of the requesting governance service (if initiated by a governance engine)
        orig_engine_name: str
            - optional unique name of the governance engine (if initiated by a governance engine).

        Returns
        -------
        Unique id (guid) of the newly started governance engine process

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        url = self.engine_command_root + "/governance-action-processes/initiate"
        body = {
            "class":                     "GovernanceActionProcessRequestBody",
            "processQualifiedName":       qualified_name,
            "requestSourceGUIDs":         request_source_guids,
            "actionTargets":              action_targets,
            "startTime":                  int(start_time.timestamp() * 1000),
            "requestParameters":          request_parameters,
            "originatorServiceName":      orig_service_name,
            "originatorEngineName":       orig_engine_name
        }
        new_body = body_slimmer(body)
        response = self.make_request("POST", url, new_body)

        return response.json().get('guid')
        
    def initiate_engine_action(self, qualified_name: str, domain_identifier: int, display_name: str,
                               description: str, request_source_guids: str, action_targets: str,
                               received_guards: [str], start_time: datetime, gov_engine_name: str,
                               request_type: str, request_parameters: dict, process_name: str,
                               request_src_name: str = None, originator_svc_name: str = None,
                               originator_eng_name: str = None) -> str:
        """
            Initiates an engine action with the specified parameters.

            Args:
                qualified_name (str): The qualified name of the governance action.
                domain_identifier (int): The domain identifier for the governance action.
                display_name (str): The display name of the governance action.
                description (str): The description of the governance action.
                request_source_guids (str): GUIDs of the sources initiating the request.
                action_targets (str): Targets of the governance action.
                received_guards (List[str]): List of guards received for the action.
                start_time (datetime): The start time for the governance action.
                gov_engine_name (str): The name of the governance engine associated with the action.
                request_type (str): The type of the governance action request.
                request_parameters (dict): Additional parameters for the governance action.
                process_name (str): The name of the associated governance action process.
                request_src_name (str, optional): The name of the request source. Defaults to None.
                originator_svc_name (str, optional): The name of the originator service. Defaults to None.
                originator_eng_name (str, optional): The name of the originator engine. Defaults to None.

            Returns:
                str: The GUID (Globally Unique Identifier) of the initiated governance action.

            Raises:
                InvalidParameterException: If the API response indicates an error (non-200 status code),
                this exception is raised with details from the response content.

            Note:
                The `start_time` parameter should be a `datetime` object representing the start 
                time of the governance action.


            """
        url = self.engine_command_root + ("/governance-engines/" + gov_engine_name +
                                          "/engine-actions/initiate")

        body = {
            "class":                      "GovernanceActionRequestBody",
            "qualifiedName":              qualified_name + str(int(start_time.timestamp())),
            "domainIdentifier":           domain_identifier,
            "displayName":                display_name,
            "description":                description,
            "requestSourceGUIDs":         request_source_guids,
            "actionTargets":              action_targets,
            "receivedGuards":             received_guards,
            "startTime":                  int(start_time.timestamp()*1000),
            "requestType":                request_type,
            "requestParameters":          request_parameters,
            "process_name":                process_name,
            "requestSourceName":          request_src_name,
            "originatorServiceName":      originator_svc_name,
            "originatorEngineName":       originator_eng_name
        }
        new_body = body_slimmer(body)
        response = self.make_request("POST", url, new_body)
        return response.json().get('guid')




"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



Execute engine actions.

"""
import os

import click

# from ops_config import Config, pass_config
from pyegeria import Platform
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    print_exception_response,
)

GERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get('KAFKA_ENDPOINT', 'localhost:9092')
EGERIA_PLATFORM_URL = os.environ.get('EGERIA_PLATFORM_URL', 'https://localhost:9443')
EGERIA_VIEW_SERVER = os.environ.get('VIEW_SERVER', 'view-server')
EGERIA_VIEW_SERVER_URL = os.environ.get('EGERIA_VIEW_SERVER_URL', 'https://localhost:9443')
EGERIA_INTEGRATION_DAEMON = os.environ.get('INTEGRATION_DAEMON', 'integration-daemon')
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get('EGERIA_INTEGRATION_DAEMON_URL', 'https://localhost:9443')
EGERIA_ADMIN_USER = os.environ.get('ADMIN_USER', 'garygeeke')
EGERIA_ADMIN_PASSWORD = os.environ.get('ADMIN_PASSWORD', 'secret')
EGERIA_USER = os.environ.get('EGERIA_USER', 'erinoverview')
EGERIA_USER_PASSWORD = os.environ.get('EGERIA_USER_PASSWORD', 'secret')


@click.command('stop')
@click.option('--server', default=EGERIA_VIEW_SERVER, help='Egeria metadata store to load')
@click.option('--url', default=EGERIA_VIEW_SERVER_URL, help='URL of Egeria platform to connect to')
@click.option('--userid', default=EGERIA_ADMIN_USER, help='Egeria admin user')
@click.option('--password', default=EGERIA_ADMIN_PASSWORD, help='Egeria admin password')
@click.option('--timeout', default=60, help='Number of seconds to wait')
def stop_daemon(file, server, url, userid, password, timeout):
    """Stop an engine-host daemon"""
    p_client = Platform(server, url, userid, password)
    try:
        p_client.shutdown_server()

        click.echo(f"Stopped server {server}")
    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)
    finally:
        p_client.close_session()


@click.command('start')
@click.option('--server', default=EGERIA_VIEW_SERVER, help='Egeria metadata store to load')
@click.option('--url', default=EGERIA_VIEW_SERVER_URL, help='URL of Egeria platform to connect to')
@click.option('--userid', default=EGERIA_ADMIN_USER, help='Egeria admin user')
@click.option('--password', default=EGERIA_ADMIN_PASSWORD, help='Egeria admin password')
@click.option('--timeout', default=60, help='Number of seconds to wait')
def start_daemon(file, server, url, userid, password, timeout):
    """Start or restart an engine-host from its known configuration """
    p_client = Platform(server, url, userid, password)
    try:
        p_client.activate_server_stored_config()

        click.echo(f"Started server {server}")

    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)
    finally:
        p_client.close_session()
